from django.conf.urls import patterns, url

from . import views

urlpatterns = patterns('',
                       url(r'^$', views.TopicList.as_view(), name='topics'),
                       url(r'^new_topic/$', views.TopicCreate.as_view(), name='new_topic'),
                       url(r'^(?P<topic_slug>[-\w]+)/$', views.PostList.as_view(), name='topic_posts'),
                       url(r'^(?P<slug>[-\w]+)/edit/$', views.TopicUpdate.as_view(), name='topic_update'),
                       url(r'^(?P<slug>[-\w]+)/delete/$', views.TopicDelete.as_view(), name='topic_delete'),
                       url(r'^(?P<topic_slug>[-\w]+)/new_post/$', views.PostCreate.as_view(), name='new_post'),
                       url(r'^(?P<topic_slug>[-\w]+)/(?P<slug>[-\w]+)/$', views.PostDetail.as_view(), name='post'),
                       url(r'^(?P<topic_slug>[-\w]+)/(?P<slug>[-\w]+)/delete/$', views.PostDelete.as_view(),
                           name='post_delete'),
                       url(r'^(?P<topic_slug>[-\w]+)/(?P<post_slug>[-\w]+)/post/$', views.post_comment,
                           name='post_comment'),
                       url(r'^(?P<topic_slug>[-\w]+)/(?P<post_slug>[-\w]+)/(?P<reply_to_slug>[-\w]+)/reply/$',
                           views.comment_reply,
                           name='comment_reply'),
                       url(r'^(?P<topic_slug>[-\w]+)/(?P<post_slug>[-\w]+)/delete_reply/(?P<reply_id>[-\w]+)/$',
                           views.delete_reply,
                           name='delete_reply')

                       )



# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse

from autoslug import AutoSlugField
from mptt.models import MPTTModel, TreeForeignKey


class Topic(models.Model):
    title = models.CharField(max_length=60)
    description = models.CharField(max_length=255, blank=True, null=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    modified = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now=True)
    slug = AutoSlugField(unique=True, populate_from='title')
    closed = models.BooleanField(blank=True, default=False, help_text='Only you can post in this Topic, '
                                                                      'Others can still reply to posts)')

    def __str__(self):     # TODO: order these
        return self.title.title()

    def get_absolute_url(self):
        return reverse('forum:topic_posts', args=[self.slug])

    @property # TODO: Whats this?
    def post_count(self):
        return self.post_set.count()

    def last_post(self):
        return self.post_set.last()


class Post(models.Model):
    title = models.CharField(max_length=60)
    body = models.TextField(max_length=1000, blank=True, null=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    modified = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now=True)
    slug = AutoSlugField(unique=True, populate_from='title')
    closed = models.BooleanField(blank=True, default=False)
    topic = models.ForeignKey(Topic, null=True)

    def __str__(self):
        return self.title.title()

    @property
    def reply_count(self):
        return self.reply_set.count()

    @property
    def last_reply(self):
        return self.reply_set.last()

    def get_absolute_url(self):
        if self.topic:
            topic_slug = self.topic.slug
        else:
            topic_slug = None
        return reverse('forum:post', args=[topic_slug, self.slug])


class Reply(MPTTModel):
    body = models.TextField(max_length=10000)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    post = models.ForeignKey(Post)
    topic = models.ForeignKey(Topic)
    user_ip = models.GenericIPAddressField(blank=True, null=True)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children', db_index=True)
    slug = AutoSlugField(unique=True, populate_from='creator')

    def __str__(self):
        return "%s - %s" % (self.creator, self.post)

    def short(self):
        return "%s \n%s" % (self.creator, self.created.strftime("%b %d, %I:%M %p"))

    short.allow_tags = True

    def get_absolute_url(self):
        return reverse('forum:post', args=[self.post.slug])

    class MPTTMeta:
        order_insertion_by = ['created']


class ProfaneWord(models.Model): # TODO: necessary?
    word = models.CharField(max_length=60)

    def __str__(self):
        return self.word



from django.contrib import admin

from .models import Topic, Post, Reply, ProfaneWord
# TODO: update

class TopicInline(admin.TabularInline):
    model = Topic
    extra = 0


class PostInline(admin.TabularInline):
    model = Post
    extra = 0


class ReplyInline(admin.TabularInline):
    model = Reply
    extra = 0


class TopicAdmin(admin.ModelAdmin):
    list_display = ["title", "post_count", "creator", "created"]
    list_filter = ["creator", "created"]
    inlines = [PostInline]


class PostAdmin(admin.ModelAdmin):
    list_display = ["title", 'topic', "reply_count", "creator", "created"]
    list_filter = ["creator", "created"]
    inlines = [ReplyInline]


class ReplyAdmin(admin.ModelAdmin):
    search_fields = ["body", "creator"]
    list_display = ["body", "post", "creator", "created"]


class ProfaneWordAdmin(admin.ModelAdmin):
    pass


admin.site.register(Topic, TopicAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Reply, ReplyAdmin)
admin.site.register(ProfaneWord, ProfaneWordAdmin)



from django import forms

from djorum.models import Reply, ProfaneWord

# class PostForm(forms.ModelForm):
#
#     title = forms.CharField(max_length=60, required=True)
#
#     class Meta():
#         model = Post
#         exclude = ('creator','updated', 'created', 'closed', 'topic',)

class ReplyForm(forms.ModelForm):
    class Meta:
        model = Reply
        exclude = ('creator', 'updated', 'created', 'topic', 'post', 'user_ip', 'parent')

    def clean_body(self):
        body = self.cleaned_data["body"]

        # TODO: hmm
        # if FORUM_FILTER_PROFANE_WORDS:
        #     profane_words = ProfaneWord.objects.all()
        #     bad_words = [w for w in profane_words if w.word in body.lower()]
        #
        #     if bad_words:
        #         raise forms.ValidationError("Bad words like '%s' are not allowed in posts." %
        #                                     (reduce(lambda x, y: "%s,%s" % (x, y), bad_words)))

        return body



from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.views import generic
from django.core.urlresolvers import reverse_lazy
from django.core.exceptions import PermissionDenied
from django.contrib import messages
from django.contrib.auth import get_user_model
# TODO: Remove user passes test mixin
from braces.views import UserPassesTestMixin

from djorum.models import Topic, Post, Reply
from djorum.forms import ReplyForm
User = get_user_model()


class TopicList(generic.ListView):
    context_object_name = 'topics'
    model = Topic
    template_name = 'djorum/index.html'


class TopicCreate(generic.CreateView):
    model = Topic
    fields = ['title', 'description', 'closed']
    template_name = 'djorum/new_topic.html'

    def form_valid(self, form):
        form = form.save(commit=False)
        form.creator = User.objects.get(id=self.request.user.id)
        form.save()
        topic = Topic.objects.get(slug=form.slug)
        messages.success(self.request, 'The topic %s was created!' % (topic,))
        return HttpResponseRedirect(reverse('djorum:topic_posts', args=(topic.slug,)))


class TopicUpdate(UserPassesTestMixin, generic.UpdateView):
    model = Topic
    fields = ['title', 'description', 'closed']
    template_name = 'djorum/topic_update_form.html'

    def form_valid(self, form):
        topic = Topic.objects.get(slug=self.kwargs['slug'])
        form = form.save(commit=False)
        if form.closed:
            messages.error(self.request, 'You have closed the topic %s!' % (topic,))
        if topic.closed and not form.closed:
            messages.success(self.request, 'You have reopened %s!' % (topic,))
        messages.success(self.request, 'The topic %s was updated!' % (topic,))
        form.save()
        return HttpResponseRedirect(reverse('djorum:topic_posts', args=(topic.slug,)))

    # TODO: test these 403s like with tribes
    def test_func(self, user):
        if Topic.objects.get(slug=self.kwargs['slug']).creator == user:
            return True
        else:
            raise PermissionDenied


class TopicDelete(UserPassesTestMixin, generic.DeleteView):
    model = Topic
    template_name = 'djorum/topic_confirm_delete.html'

    success_url = reverse_lazy('djorum:topics')

    def test_func(self, user):
        if Topic.objects.get(slug=self.kwargs['slug']).creator == user:
            return True
        else:
            raise PermissionDenied


class PostList(generic.ListView):
    context_object_name = 'posts'
    model = Post
    template_name = 'djorum/post_list.html'

    # TODO: investigate if this does anything
    def get_queryset(self):
        topic = Topic.objects.get(slug=self.kwargs['topic_slug'])
        return Post.objects.filter(topic=topic)

    def get_context_data(self, **kwargs):
        context = super(PostList, self).get_context_data(**kwargs)
        topic = Topic.objects.get(slug=self.kwargs['topic_slug'])
        context['topic'] = topic
        return context


class PostCreate(generic.CreateView):
    model = Post
    fields = ['title', 'body', ]
    template_name = 'djorum/new_post.html'

    def form_valid(self, form):
        form = form.save(commit=False)
        form.creator = User.objects.get(id=self.request.user.id)
        form.topic = Topic.objects.get(slug=self.kwargs['topic_slug'])
        form.save()
        post = Post.objects.get(slug=form.slug)
        messages.success(self.request, 'Post successful!')
        return HttpResponseRedirect(reverse('djorum:post', args=(post.topic.slug, post.slug,)))

    def get_context_data(self, **kwargs):
        context = super(PostCreate, self).get_context_data(**kwargs)
        topic = Topic.objects.get(slug=self.kwargs['topic_slug'])
        context['topic'] = topic
        return context


class PostDelete(UserPassesTestMixin, generic.DeleteView):
    model = Post
    template_name = 'djorum/post_confirm_delete.html'

    def get_success_url(self, *args, **kwargs):
        return reverse('djorum:topic_posts', args=(self.kwargs['topic_slug'],))

    def test_func(self, user):
        if Post.objects.get(slug=self.kwargs['slug']).creator == user:
            return True
        else:
            raise PermissionDenied


class PostDetail(generic.DetailView):
    model = Post
    template_name = 'djorum/post.html'

    def get_context_data(self, **kwargs):
        context = super(PostDetail, self).get_context_data(**kwargs)
        post = Post.objects.get(slug=self.kwargs['slug'])
        context['post'] = post
        replies = Reply.objects.filter(post=post)
        context['replies'] = replies
        context['reply_form'] = ReplyForm()
        return context


def post_comment(request, topic_slug, post_slug):
    target_post = Post.objects.get(slug=post_slug)

    if request.method == 'POST':
        form = ReplyForm(request.POST)

        if form.is_valid():
            comment = Reply()
            comment.post = target_post
            comment.topic = target_post.topic
            comment.body = form.cleaned_data['body']
            comment.creator = request.user
            comment.user_ip = request.META['REMOTE_ADDR']

            comment.save()
            messages.success(request, 'Reply successful!')

            return HttpResponseRedirect(reverse('djorum:post', args=(topic_slug, post_slug)))


def comment_reply(request, topic_slug, post_slug, reply_to_slug):
    target_post = Post.objects.get(slug=post_slug)
    reply_to = Reply.objects.get(slug=reply_to_slug)

    if request.method == 'POST':
        form = ReplyForm(request.POST)

        if form.is_valid():
            reply = Reply()
            reply.post = target_post
            reply.topic = target_post.topic
            reply.body = form.cleaned_data['body']
            reply.creator = request.user
            reply.user_ip = request.META['REMOTE_ADDR']
            reply.parent = reply_to

            reply.save()
            messages.success(request, 'Reply successful!')

            return HttpResponseRedirect(reverse('djorum:post', args=(topic_slug, post_slug)))


def delete_reply(request, topic_slug, post_slug, reply_id):
    reply = Reply.objects.get(id=reply_id)

    if request.method == "POST":

        if request.user == reply.creator or request.user == reply.topic.creator:
            reply.delete()
            messages.success(request, 'Reply %s deleted!' % reply)
        return HttpResponseRedirect(request.GET.get('next'))
    else:
        messages.error(request, 'error')
        return HttpResponseRedirect(reverse('djorum:post', args=(topic_slug, post_slug)))
